package asd.td4;

import asd.datatype.PileStatique;
import asd.datatype.Entier;

/**
 * Une classe pour abstraire un chiquier ne contenant que des reines libres
 * (c'est  dire qui ne sont pas en prise les unes avec les autres). Pour
 * un chiquier  n cases, les lignes et les colonnes sont numrotes de
 * 0  n - 1 et les reines sont numrotes de 0  n - 1, la reine de 
 * numro i tant obligatoirement dans la colonne numro i.
 */
public class EchiquierDeReines {

    private PileStatique pile;   // Si pile = [l0,l1,..,lk[ ou lk est le sommet
                                 // de pile, alors les reines 0,1,..,k sont
                                 // places sur les lignes l0,l1,..,lk .
    private GestionOccupationEchiquier maGOE; // L'chiquier qui conserve la trace du
                                 // placement des reines.

    /**
     * Construit un chiquier de reines vide (i.e. ne contenant aucune
     * reine).<br>
     * @param n la taille de l'chiquier et le nombre de reines  placer.<br>
     * Prcondition : n >= 1.<br>
     * Complexit : O(1).
     */
    public EchiquierDeReines(int n) {
        pile = new PileStatique(n);
        maGOE = new GestionOccupationEchiquier(n);
    }

    /**
     * Teste si l'chiquier est plein, c'est  dire s'il contient n reines
     * libres, o n est la taille de l'chiquier.<br>
     * Valeur rendue : true si l'chiquier contient n reines libres, o
     * n est la taille de l'chiquier.<br>
     * Complexit : O(1).
     */
    public boolean plein() {
        return pile.hauteur() == maGOE.taille();
    }

    /**
     * Teste si l'chiquier est vide, c'est  dire s'il ne contient aucune
     * reine.<br>
     * Valeur rendue : true si l'chiquier est vide (i.e. ne contient aucune
     * reine).<br>
     * Complexit : O(1).
     */
    public boolean vide() {
        return pile.vide();
    }

    /**
     * Teste si une nouvelle reine est en prise avec les reines dj
     * places sur l'chiquier.<br>
     * @param l un numro de ligne.<br>
     * Prcondition : 0 <= l < n, l'chiquier contient les k reines libres
     * 0,1,..,k-1 et k < n.<br>
     * Valeur rendue : true si, place en ligne l et en colonne k, la reine
     * k est en prise avec d'autres reines de l'chiquier.<br>
     * Complexit : O(1).
     */
    public boolean enPrise(int l) {
        return maGOE.enPrise(l,pile.hauteur());
    }

    /**
     * Ajoute une nouvelle reine libre  l'chiquier.<br>
     * @param l un numro de ligne.<br>
     * Prcondition : 0 <= l < n, l'chiquier contient les k reines libres
     * 0,1,..,k-1 avec k < n et, place en ligne l et en colonne k, la reine
     * k n'est en prise avec aucune autre reine de l'chiquier.<br>
     * Postrelation : l'chiquier contient les k+1 reines libres 0,1,..,k et
     * la reine k est en ligne l.<br>
     * Complexit : O(1).
     */
    public void ajouterNouvelleReine(int l) {
        maGOE.ajouter(l,pile.hauteur());
        pile.empiler(new Entier(l));
    }

    /**
     * Enlve la dernire reine qui a t place sur l'chiquier.<br>
     * Prcondition : l'chiquier n'est pas vide et contient les k reines
     * libres 0,1,..,k-1 avec 1 <= k <= n.<br>
     * Postrelation : si k > 1, l'chiquier contient les k-1 reines libres
     * 0,1,..,k-2 sinon l'chiquier est vide.<br>
     * Complexit : O(1).
     */
    public void enleverDerniereReine() {
        int ligne = ((Entier) pile.sommet()).intValue();
        maGOE.enlever(ligne,pile.hauteur()-1);
        pile.depiler();
    }

    /**
     * Retourne le numro de la ligne de la dernire reine qui a t
     * place sur l'chiquier.<br>
     * Prcondition : l'chiquier n'est pas vide et contient les k reines
     * libres 0,1,..,k-1 avec 1 <= k <= n.<br>
     * Valeur rendue : le numro de ligne de la reine k-1.<br>
     * Complexit : O(1).
     */
    public int derniereLigne() {
        return ((Entier) pile.sommet()).intValue();
    }

    /**
     * Construit une chane de caractres reprsentant l'chiquier.<br>
     * Valeur rendue : une chane de caractres reprsentant l'chiquier.<br>
     * Complexit : O(k) o k est le nombre de reines places sur l'chiquier.
     */
    public String toString() {
        if ( pile.vide() ) {
            return "PAS DE SOLUTION";
        }
        String s = "";
        PileStatique p = (PileStatique) pile.clone();
        int k = pile.hauteur();
        while ( ! pile.vide() ) {
            s = "reine " + k + " en ligne " + (derniereLigne()+1) + " et en colonne " + k + "\n" + s;
            k--;
            pile.depiler();
        }
        pile = p;
        return s;
    }
}
