package asd.td4;

/**
 * Une classe pour un chiquier qui garde les informations pertinentes
 * concernant le placement de reines. Pour un chiquier  n cases, les
 * lignes et les colonnes sont numrotes de 0  n - 1.
 */
public class Echiquier {

    private boolean[] lignes;       // les lignes
    private boolean[] montantes;    // les diagonales montantes
    private boolean[] descendantes; // les diagonales descendantes

    /**
     * Construit un chiquier de reines vide (i.e. ne contenant aucune
     * reine).<br>
     * @param n la taille de l'chiquier.<br>
     * Prcondition : n >= 1.<br>
     * Complexit : O(1).
     */
    public Echiquier(int n) {
        lignes = new boolean[n];
        montantes = new boolean[2*n - 1];
        descendantes = new boolean[2*n - 1];
    }

    /**
     * Retourne la taille de l'chiquier.<br>
     * Complexit : O(1).
     */
    public int taille() {
        return lignes.length;
    }

    /**
     * Teste si on peut ajouter une reine libre  une position donne.<br>
     * @param l la ligne o on dsire ajouter la reine.
     * @param c la colonne o on dsire ajouter la reine.<br>
     * Prcondition : 0 <= l < n et 0 <= c < n o n est la taille de
     * l'chiquier.<br>
     * Valeur rendue : true si une nouvelle reine place en ligne l et en
     * colonne c est en prise avec d'autres reines dj places sur
     * l'chiquier.<br>
     * Complexit : O(1).
     */
    public boolean enPrise(int l, int c) {
        return lignes[l] || montantes[l + c] ||
               descendantes[l - c + lignes.length - 1];
    }

    /**
     * Ajoute une reine sur l'chiquier  une position donne.<br>
     * @param l la ligne o on dsire ajouter la reine.
     * @param c la colonne o on dsire ajouter la reine.<br>
     * Prcondition : 0 <= l < n et 0 <= c < n o n est la taille de
     * l'chiquier.<br>
     * Postrelation : l'chiquier contient une nouvelle reine  la ligne l
     * et la colonne c.<br>
     * Complexit : O(1).
     */
    public void ajouter(int l, int c) {
        lignes[l] = true;
        montantes[l + c] = true;
        descendantes[l - c + lignes.length - 1] = true;
    }

    /**
     * Enlve une reine sur l'chiquier  une position donne.<br>
     * @param l la ligne de la reine qu'on dsire enlever.
     * @param c la colonne de la reine qu'on dsire enlever.<br>
     * Prcondition : 0 <= l < n et 0 <= c < n o n est la taille de
     * l'chiquier.<br>
     * Postrelation : l'chiquier ne contient plus de reine  la ligne l
     * et la colonne c.<br>
     * Complexit : O(1).
     */
    public void enlever(int l, int c) {
        lignes[l] = false;
        montantes[l + c] = false;
        descendantes[l - c + lignes.length - 1] = false;
    }
}

    
